'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

require('process');
var React = require('react');
var React__default = _interopDefault(React);

var SET_INPUT = 'SET_INPUT';
var START_LOADING = 'START_LOADING';
var END_LOADING = 'END_LOADING';
var SEARCH_DATA = 'SEARCH_DATA';
var DEBOUNCE = 'DEBOUNCE';
var THROTTLE = 'THROTTLE';
var RESET_STATE = 'RESET_STATE';
var STRING_MATCHING = 'STRING_MATCHING';
var CHARACTER_MATCHING = 'CHARACTER_MATCHING';
var DEFAULT = 'DEFAULT';
var POPOVER_EXPANDED = 'POPOVER_EXPANDED';
var __DEV__ =  "development" === 'development';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var reducer = function reducer(state, action) {
  switch (action.type) {
    case SET_INPUT:
      return _extends({}, state, {
        input: action.payload
      });

    case SEARCH_DATA:
      return _extends({}, state, {
        searchData: action.payload
      });

    case START_LOADING:
      return _extends({}, state, {
        isLoading: true
      });

    case END_LOADING:
      return _extends({}, state, {
        isLoading: false
      });

    case RESET_STATE:
      return _extends({}, initialState);

    default:
      return state;
  }
};

var isMatch = function isMatch(obj, _char, keys) {
  return keys.some(function (match) {
    return obj[match].toLowerCase().includes(_char);
  });
};
var replaceAll = function replaceAll(obj, regex, keys) {
  var newObj = _extends({}, obj);

  keys.forEach(function (key) {
    return newObj[key] = obj[key].replaceAll(regex, function (match) {
      return "<mark>" + match + "<mark>";
    });
  });
  return newObj;
};
var isResultsNotFound = function isResultsNotFound(searchData, searchInput) {
  return searchData.length === 0 && searchInput.length > 0;
};

var initialState = {
  isLoading: false,
  searchData: [],
  input: ''
};
var context = /*#__PURE__*/React.createContext(null);
/**
 * @returns context data and helper function
 */

var useReactSearchHighlight = function useReactSearchHighlight() {
  return React.useContext(context);
};
var ReactSearchHighlightProvider = function ReactSearchHighlightProvider(_ref) {
  var children = _ref.children;

  var _useReducer = React.useReducer(reducer, initialState),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  var startLoading = function startLoading() {
    dispatch({
      type: START_LOADING
    });
  };

  var endLoading = function endLoading() {
    dispatch({
      type: END_LOADING
    });
  };

  var resetState = function resetState() {
    dispatch({
      type: RESET_STATE
    });
  };

  var isResultsEmpty = isResultsNotFound(state.searchData, state.input);
  var value = React.useMemo(function () {
    return {
      suggestions: state.searchData,
      isLoading: state.isLoading,
      input: state.input,
      dispatch: dispatch,
      startLoading: startLoading,
      endLoading: endLoading,
      resetState: resetState,
      isResultsEmpty: isResultsEmpty
    };
  }, [state]);
  return React__default.createElement(context.Provider, {
    value: value
  }, children);
};

var _excluded = ["as", "children", "classNames"];

var Text = function Text(_ref) {
  var as = _ref.as,
      children = _ref.children,
      classNames = _ref.classNames,
      any = _objectWithoutPropertiesLoose(_ref, _excluded);

  return React.createElement(as, _extends({
    className: classNames
  }, any), children);
};

/**
 * @param keys - keys to search for from available data
 * @returns {callback} - dispatches action to update search results after character strings
 */

var useCharacterMatching = function useCharacterMatching(keys) {
  var _useReactSearchHighli = useReactSearchHighlight(),
      dispatch = _useReactSearchHighli.dispatch;

  return function (input, data) {
    var inputArr = input.toLowerCase().split('');

    if (inputArr.length === 0) {
      dispatch == null ? void 0 : dispatch({
        type: SEARCH_DATA,
        payload: []
      });
      return;
    }

    var regex = new RegExp(inputArr.join('|'), 'gi');

    if (!Array.isArray(data) || !Array.isArray(keys)) {
      {
        console.error("Element type is invalid: expected a array but got: " + typeof data + "." + 'This could happen because you have passed wrong data type in data props');
      }

      return;
    }

    if (data.length > 0 && !Object.keys(data[0]).some(function (key) {
      return keys.includes(key);
    })) {
      {
        console.error('Keys are invalid: cannot perform matching operation. \n' + 'This could happen for one of the following reasons:\n' + '\t1. keysToSearch does not contain any key for which data object has valid value' + '\t2. You might have passed wrong data types in keysToSearch props\n');
      }

      return;
    }

    var newArr = data == null ? void 0 : data.filter(function (single) {
      return inputArr.every(function (inputChar) {
        return isMatch(single, inputChar, keys);
      });
    }).map(function (item) {
      var newItem = replaceAll(item, regex, keys);
      return _extends({}, item, newItem);
    });
    dispatch == null ? void 0 : dispatch({
      type: SEARCH_DATA,
      payload: newArr
    });
  };
};

var useDebounce = function useDebounce(value, delay) {
  var _useState = React.useState(value),
      debouncedValue = _useState[0],
      setDebouncedValue = _useState[1];

  React.useEffect(function () {
    var handler = setTimeout(function () {
      setDebouncedValue(value);
    }, delay);
    return function () {
      clearTimeout(handler);
    };
  }, [value, delay]);
  return debouncedValue;
};

/**
 * Triggers callback function when deps update skipping initial mount
 * @param deps dependency array for useEffect hook
 * @param func callback - triggered on keydown event
 */

var useDidMountEffect = function useDidMountEffect(func, deps) {
  var didMount = React.useRef(false);
  React.useEffect(function () {
    if (didMount.current) func();else didMount.current = true;
  }, deps);
};

/**
 * Triggers callback function on keydown while holding cmd
 * @param callback function - triggered on keydown event
 * @param withCmd boolean - with command key pressed
 * @param eventKey string - keydown event key
 * @param deps array - dependencies
 */

var useKeyDown = function useKeyDown(callback, withCmd, eventKey, deps) {
  if (eventKey === void 0) {
    eventKey = 'k';
  }

  if (deps === void 0) {
    deps = [];
  }

  React.useEffect(function () {
    var keydownCb = function keydownCb(event) {
      var key = event['key'];
      var cmdHeld = withCmd ? event['metaKey'] : true;

      if (cmdHeld && key.toLowerCase() === eventKey) {
        callback == null ? void 0 : callback();
      }
    };

    document.addEventListener('keydown', keydownCb);
    return function () {
      document.removeEventListener('keydown', keydownCb);
    };
  }, deps);
};

var useOffScreen = function useOffScreen(ref) {
  var _useState = React.useState(false),
      state = _useState[0],
      setState = _useState[1];

  React.useEffect(function () {
    function handleClickEvent(event) {
      if (ref.current && !ref.current.contains(event.target)) setState(false);
    }

    document.addEventListener('mousedown', handleClickEvent);
    return function () {
      document.removeEventListener('mousedown', handleClickEvent);
    };
  }, [ref]);
  return [state, setState];
};

/**

 * useRefComposition combines multiple refs into a single composed ref
 * @param refs - Array of refs
 * @returns callback function to update ref

 */
var useRefComposition = function useRefComposition(refs) {
  return function (el) {
    refs.forEach(function (ref) {
      if (ref === null) return;
      if (typeof ref === 'function') ref(el);else ref.current = el;
    });
  };
};

/**
 * @param keys - keys to search for from available data
 * @returns {callback} - dispatches action to update search results after matching strings
 */

var useStringMatching = function useStringMatching(keys) {
  var _useReactSearchHighli = useReactSearchHighlight(),
      dispatch = _useReactSearchHighli.dispatch;

  return function (input, data) {
    if (input.length === 0) {
      dispatch == null ? void 0 : dispatch({
        type: SEARCH_DATA,
        payload: []
      });
      return;
    }

    var regex = new RegExp(input, 'gi');

    if (!Array.isArray(data) || !Array.isArray(keys)) {
      {
        console.error("Element type is invalid: expected a array but got: " + typeof data + "." + 'This could happen because you have passed wrong data type in data props');
      }

      return;
    }

    if (data.length > 0 && !Object.keys(data[0]).some(function (key) {
      return keys.includes(key);
    })) {
      {
        console.error('Keys are invalid: cannot perform matching operation. \n' + 'This could happen for one of the following reasons:\n' + '\t1. keysToSearch does not contain any key for which data object has valid value' + '\t2. You might have passed wrong data types in keysToSearch props\n');
      }

      return;
    }

    var newArr = data == null ? void 0 : data.filter(function (single) {
      return isMatch(single, input, keys);
    }).map(function (item) {
      var newItem = replaceAll(item, regex, keys);
      return _extends({}, item, newItem);
    });
    dispatch == null ? void 0 : dispatch({
      type: SEARCH_DATA,
      payload: newArr
    });
  };
};

var useThrottle = function useThrottle(value, ms) {
  if (ms === void 0) {
    ms = 200;
  }

  var _useState = React.useState(value),
      state = _useState[0],
      setState = _useState[1];

  var timeout = React.useRef();
  var nextValue = React.useRef(null);
  var hasNextValue = React.useRef(0);
  var fnRef = React.useRef(null);
  React.useEffect(function () {
    if (!timeout.current) {
      setState(value);

      var timeoutCallback = function timeoutCallback() {
        if (hasNextValue.current) {
          hasNextValue.current = false;
          setState(nextValue.current);
          timeout.current = setTimeout(timeoutCallback, ms);
        } else {
          timeout.current = undefined;
        }
      };

      timeout.current = setTimeout(timeoutCallback, ms);
    } else {
      nextValue.current = value;
      hasNextValue.current = true;
    }
  }, [value]);

  var fn = function fn() {
    timeout.current && clearTimeout(timeout.current);
  };

  fnRef.current = fn;
  React.useEffect(function () {
    return function () {
      return fnRef.current();
    };
  }, []);
  return state;
};

var _excluded$1 = ["children"],
    _excluded2 = ["children", "optionIndex"],
    _excluded3 = ["className", "value", "as"];
var InternalContextInitialState = {
  isPopoverExpanded: false,
  listItemActiveIndex: 0,
  updateInternalContext: function updateInternalContext(key, value) {},
  dataLength: 0
};
var InternalContext = /*#__PURE__*/React.createContext(InternalContextInitialState);
var WrapperInner = /*#__PURE__*/React__default.forwardRef(function (_ref, forwardedRef) {
  var children = _ref.children,
      isFunction = _ref.isFunction;

  var _useState = React.useState(InternalContextInitialState),
      state = _useState[0],
      setState = _useState[1];

  var __state = useReactSearchHighlight();

  var updateInternalContext = function updateInternalContext(key, value) {
    setState(function (prev) {
      var _extends2;

      return _extends({}, prev, (_extends2 = {}, _extends2[key] = value, _extends2));
    });
  };

  var listRef = React.useRef(null);
  var composedRefs = useRefComposition([listRef, forwardedRef]);

  var _useOffScreen = useOffScreen(listRef),
      isListVisible = _useOffScreen[0],
      setIsListVisible = _useOffScreen[1];

  useDidMountEffect(function () {
    updateInternalContext('isPopoverExpanded', isListVisible);
  }, [isListVisible]);
  return React__default.createElement(InternalContext.Provider, {
    value: _extends({}, state, {
      updateInternalContext: updateInternalContext
    })
  }, React__default.createElement("section", {
    ref: composedRefs,
    className: "rsh-search-wrapper",

    /**
     * onFocusCapture is triggered when input(child) is focused
     */
    onFocusCapture: function onFocusCapture() {
      return setIsListVisible(true);
    }
  }, isFunction ? children(__state) : children));
});
var Wrapper = /*#__PURE__*/React__default.forwardRef(function (_ref2, forwardedRef) {
  var children = _ref2.children;
  var isFunction = typeof children === 'function';
  return React__default.createElement(React__default.Fragment, null, isFunction ? React__default.createElement(ReactSearchHighlightProvider, null, React__default.createElement(WrapperInner, {
    ref: forwardedRef,
    isFunction: isFunction,
    children: children
  })) : React__default.createElement(WrapperInner, {
    ref: forwardedRef,
    isFunction: isFunction,
    children: children
  }));
});
var PopOverList = /*#__PURE__*/React__default.forwardRef(function (_ref3, forwardedRef) {
  var children = _ref3.children,
      any = _objectWithoutPropertiesLoose(_ref3, _excluded$1);

  var _useReactSearchHighli = useReactSearchHighlight(),
      suggestions = _useReactSearchHighli.suggestions;

  var _React$useContext = React__default.useContext(InternalContext),
      isPopoverExpanded = _React$useContext.isPopoverExpanded,
      listItemActiveIndex = _React$useContext.listItemActiveIndex,
      updateInternalContext = _React$useContext.updateInternalContext;

  var listItemActiveIndexArrowDown = function listItemActiveIndexArrowDown() {
    if (listItemActiveIndex < suggestions.length - 1) updateInternalContext('listItemActiveIndex', listItemActiveIndex + 1);
  };

  var listItemActiveIndexArrowUp = function listItemActiveIndexArrowUp() {
    if (listItemActiveIndex > 0) updateInternalContext('listItemActiveIndex', listItemActiveIndex - 1);
  };

  useKeyDown(listItemActiveIndexArrowDown, false, 'arrowdown', [listItemActiveIndex, suggestions.length]);
  useKeyDown(listItemActiveIndexArrowUp, false, 'arrowup', [listItemActiveIndex]);
  return React__default.createElement(React__default.Fragment, null, isPopoverExpanded && React__default.createElement("ul", Object.assign({
    "data-popover-expanded": isPopoverExpanded,
    ref: forwardedRef,
    className: "rsh-search-list"
  }, any), children));
});
var PopOverOption = function PopOverOption(_ref4) {
  var children = _ref4.children,
      optionIndex = _ref4.optionIndex,
      any = _objectWithoutPropertiesLoose(_ref4, _excluded2);

  var _React$useContext2 = React__default.useContext(InternalContext),
      updateInternalContext = _React$useContext2.updateInternalContext,
      listItemActiveIndex = _React$useContext2.listItemActiveIndex;

  if ( typeof optionIndex !== 'number') throw new ReferenceError("optionIndex type is invalid: expected a number but got: " + typeof optionIndex + ". " + "To avoid the error please provide optionIndex in PopOverOption component" + "\n\t{state.searchData?.map((item, index) => (" + "\n\t\t\t<PopOverOption" + "\n\t\t\t\toptionIndex={index}");

  var handleMouserEnter = function handleMouserEnter() {
    updateInternalContext('listItemActiveIndex', optionIndex);
  };

  return React__default.createElement("li", Object.assign({
    onMouseEnter: handleMouserEnter,
    className: "rsh-search-list-item " + (listItemActiveIndex === optionIndex && 'rsh-search-list-item__active')
  }, any), children);
};
var PopOverOptionText = function PopOverOptionText(_ref5) {
  var className = _ref5.className,
      value = _ref5.value,
      _ref5$as = _ref5.as,
      as = _ref5$as === void 0 ? 'h3' : _ref5$as,
      any = _objectWithoutPropertiesLoose(_ref5, _excluded3);

  var _useReactSearchHighli2 = useReactSearchHighlight(),
      input = _useReactSearchHighli2.input;

  var _React$useContext3 = React__default.useContext(InternalContext),
      matchingAlgorithm = _React$useContext3.matchingAlgorithm;

  var words = value == null ? void 0 : value.split(/<mark>/);

  var isHighlighted = function isHighlighted(word) {
    return matchingAlgorithm === STRING_MATCHING ? word.toLocaleLowerCase() === input : input.includes(word.toLocaleLowerCase());
  };

  return React__default.createElement(Text, Object.assign({
    as: as,
    classNames: className + " rsh-search-list-item-text"
  }, any), words == null ? void 0 : words.map(function (word, index) {
    return isHighlighted(word) ? React__default.createElement("span", {
      key: word + "-" + index,
      "data-user-value": true,
      "data-suggested-value": true,
      "data-heightlighted-value": true
    }, word) : React__default.createElement("span", {
      key: word + "-" + index
    }, word);
  }));
};

var Modal = function Modal(_ref) {
  var children = _ref.children,
      isOpen = _ref.isOpen,
      onClose = _ref.onClose;

  var __modal = isOpen != null ? isOpen : true;

  return React__default.createElement(React__default.Fragment, null, __modal && React__default.createElement("div", {
    onClick: onClose != null ? onClose : undefined,
    className: "rsh-modal-dialog",
    style: {
      backgroundColor: 'rgba(220,220,220,0.5)',
      position: 'fixed',
      top: 0,
      right: 0,
      bottom: 0,
      left: 0,
      zIndex: 1000
    }
  }, React__default.createElement("div", {
    className: "rsh-modal-dialog-box-wrapper",
    style: {
      maxWidth: 400,
      margin: 'auto',
      padding: 5
    }
  }, children)));
};

var _excluded$2 = ["as", "children", "className", "onClick", "onMouseDown", "onFocusCapture"];
var Stack = function Stack(_ref) {
  var as = _ref.as,
      children = _ref.children,
      className = _ref.className,
      onClick = _ref.onClick,
      onFocusCapture = _ref.onFocusCapture,
      args = _objectWithoutPropertiesLoose(_ref, _excluded$2);

  return React__default.createElement("div", {
    style: _extends({
      display: 'flex',
      flexDirection: as === 'HStack' ? 'row' : 'column',
      alignItems: 'center'
    }, args),
    className: className,
    onClick: onClick,
    onFocusCapture: onFocusCapture
  }, children);
};

var SearchIcon = function SearchIcon() {
  return React__default.createElement("svg", {
    width: "18",
    height: "18",
    viewBox: "0 0 14 14",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, React__default.createElement("path", {
    d: "M14 13.3L9.70001 9.00001C11.6 6.70001 11.3 3.20001 9.00001 1.30001C6.70001 -0.599986 3.20001 -0.299986 1.30001 2.00001C-0.599986 4.30001 -0.299986 7.80001 2.00001 9.70001C4.00001 11.4 7.00001 11.4 9.00001 9.70001L13.3 14L14 13.3ZM1.00001 5.50001C1.00001 3.00001 3.00001 1.00001 5.50001 1.00001C8.00001 1.00001 10 3.00001 10 5.50001C10 8.00001 8.00001 10 5.50001 10C3.00001 10 1.00001 8.00001 1.00001 5.50001Z",
    fill: "black"
  }));
};

var _excluded$3 = ["keysToSearch", "inputAlgorithm", "matchingAlgorithm", "data", "value", "inputAlgorithmTimeout", "onChange", "initialValue", "PrefixIcon"];
var inputAlgorithms = {
  DEBOUNCE: useDebounce,
  THROTTLE: useThrottle,
  DEFAULT: function DEFAULT(e) {
    return e;
  }
};
var matchingAlgorithms = {
  CHARACTER_MATCHING: useCharacterMatching,
  STRING_MATCHING: useStringMatching
};
var SearchBar = /*#__PURE__*/React__default.forwardRef(function (_ref, forwardedRef) {
  var _keysToSearch;

  var keysToSearch = _ref.keysToSearch,
      _ref$inputAlgorithm = _ref.inputAlgorithm,
      inputAlgorithm = _ref$inputAlgorithm === void 0 ? DEBOUNCE : _ref$inputAlgorithm,
      _ref$matchingAlgorith = _ref.matchingAlgorithm,
      matchingAlgorithm = _ref$matchingAlgorith === void 0 ? CHARACTER_MATCHING : _ref$matchingAlgorith,
      data = _ref.data,
      controlledValue = _ref.value,
      _ref$inputAlgorithmTi = _ref.inputAlgorithmTimeout,
      inputAlgorithmTimeout = _ref$inputAlgorithmTi === void 0 ? 500 : _ref$inputAlgorithmTi,
      onChange = _ref.onChange,
      initialValue = _ref.initialValue,
      PrefixIcon = _ref.PrefixIcon,
      any = _objectWithoutPropertiesLoose(_ref, _excluded$3);

  var _useState = React.useState(initialValue != null ? initialValue : ''),
      input = _useState[0],
      setInput = _useState[1];

  var _useReactSearchHighli = useReactSearchHighlight(),
      dispatch = _useReactSearchHighli.dispatch;

  var __internalContext = React__default.useContext(InternalContext);

  keysToSearch = (_keysToSearch = keysToSearch) != null ? _keysToSearch : Object.keys(data == null ? void 0 : data[0]);
  var inputRef = React.useRef(null);
  var composedRefs = useRefComposition([inputRef, forwardedRef]);
  var searchInput = controlledValue != null ? controlledValue : input;
  var searchTerm = inputAlgorithms[inputAlgorithm](searchInput, inputAlgorithmTimeout);
  var matchingFn = matchingAlgorithms[matchingAlgorithm](keysToSearch);

  {
    if (typeof controlledValue !== 'undefined' && !onChange || typeof controlledValue === 'undefined' && onChange) {
      console.warn('SearchBar input value is changing from controlled to uncontrolled, expected a onChange and value prop or none.');
    }
  }

  React.useEffect(function () {
    if (data) matchingFn(searchInput, data);
    dispatch == null ? void 0 : dispatch({
      type: SET_INPUT,
      payload: searchInput
    });
  }, [searchTerm]);
  React.useEffect(function () {
    if (__internalContext) __internalContext.updateInternalContext('matchingAlgorithm', matchingAlgorithm);
  }, []);

  var focusInput = function focusInput() {
    var _inputRef$current;

    return inputRef == null ? void 0 : (_inputRef$current = inputRef.current) == null ? void 0 : _inputRef$current.focus();
  };

  useKeyDown(focusInput, true);

  var handleOnChange = function handleOnChange(e) {
    return setInput(e.target.value.toLowerCase());
  };

  return React__default.createElement(Stack, {
    as: "HStack",
    onClick: focusInput,
    justifyContent: 'center',
    className: "rsh-input-box",
    padding: 0,
    height: 40,
    cursor: 'text'
  }, React__default.createElement("figure", {
    className: "rsh-input-box-logo"
  }, PrefixIcon ? React__default.createElement(PrefixIcon, null) : React__default.createElement(SearchIcon, null)), React__default.createElement("input", Object.assign({
    value: controlledValue != null ? controlledValue : input,
    onChange: onChange != null ? onChange : handleOnChange,
    placeholder: "search here",
    className: 'rsh-input',
    autoFocus: true,
    ref: composedRefs
  }, any)));
});

exports.CHARACTER_MATCHING = CHARACTER_MATCHING;
exports.DEBOUNCE = DEBOUNCE;
exports.DEFAULT = DEFAULT;
exports.END_LOADING = END_LOADING;
exports.InternalContext = InternalContext;
exports.Modal = Modal;
exports.POPOVER_EXPANDED = POPOVER_EXPANDED;
exports.PopOverList = PopOverList;
exports.PopOverOption = PopOverOption;
exports.PopOverOptionText = PopOverOptionText;
exports.RESET_STATE = RESET_STATE;
exports.ReactSearchHighlightProvider = ReactSearchHighlightProvider;
exports.SEARCH_DATA = SEARCH_DATA;
exports.SET_INPUT = SET_INPUT;
exports.START_LOADING = START_LOADING;
exports.STRING_MATCHING = STRING_MATCHING;
exports.SearchBar = SearchBar;
exports.THROTTLE = THROTTLE;
exports.Wrapper = Wrapper;
exports.__DEV__ = __DEV__;
exports.context = context;
exports.initialState = initialState;
exports.inputAlgorithms = inputAlgorithms;
exports.matchingAlgorithms = matchingAlgorithms;
exports.useReactSearchHighlight = useReactSearchHighlight;
//# sourceMappingURL=react-search-highlight.cjs.development.js.map
